package com.sun.media.sound;

import javax.sound.sampled.LineUnavailableException;
import java.io.ByteArrayOutputStream;
import javax.sound.sampled.LineEvent;
import java.util.Arrays;
import javax.sound.sampled.AudioInputStream;
import java.io.IOException;
import javax.sound.sampled.DataLine;
import java.io.InputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.Clip;

public final class SoftMixingClip extends SoftMixingDataLine implements Clip
{
    private AudioFormat format;
    private int framesize;
    private byte[] data;
    private final InputStream datastream;
    private int offset;
    private int bufferSize;
    private float[] readbuffer;
    private boolean open;
    private AudioFormat outputformat;
    private int out_nrofchannels;
    private int in_nrofchannels;
    private int frameposition;
    private boolean frameposition_sg;
    private boolean active_sg;
    private int loopstart;
    private int loopend;
    private boolean active;
    private int loopcount;
    private boolean _active;
    private int _frameposition;
    private boolean loop_sg;
    private int _loopcount;
    private int _loopstart;
    private int _loopend;
    private float _rightgain;
    private float _leftgain;
    private float _eff1gain;
    private float _eff2gain;
    private AudioFloatInputStream afis;
    
    SoftMixingClip(SoftMixingMixer softMixingMixer, DataLine.Info info) {
        super(softMixingMixer, info);
        this.datastream = new InputStream() {
            @Override
            public int read() throws IOException {
                byte[] b = { 0 };
                int read = super.read(b);
                if (read < 0) {
                    return read;
                }
                return b[0] & 0xFF;
            }
            
            @Override
            public int read(byte[] array, int i, int n) throws IOException {
                if (SoftMixingClip.this._loopcount != 0) {
                    int n2 = SoftMixingClip.this._loopend * SoftMixingClip.this.framesize;
                    int n3 = SoftMixingClip.this._loopstart * SoftMixingClip.this.framesize;
                    int n4 = SoftMixingClip.this._frameposition * SoftMixingClip.this.framesize;
                    if (n4 + n >= n2 && n4 < n2) {
                        int n5;
                        int n6;
                        for (n5 = i + n, ; i != n5; i += n) {
                            if (n4 == n2) {
                                if (SoftMixingClip.this._loopcount == 0) {
                                    break;
                                }
                                n4 = n3;
                                if (SoftMixingClip.this._loopcount != -1) {
                                    SoftMixingClip.this._loopcount--;
                                }
                            }
                            n = n5 - i;
                            n6 = n2 - n4;
                            if (n > n6) {
                                n = n6;
                            }
                            System.arraycopy(SoftMixingClip.this.data, n4, array, i, n);
                        }
                        if (SoftMixingClip.this._loopcount == 0) {
                            n = n5 - i;
                            int n7 = n2 - n4;
                            if (n > n7) {
                                n = n7;
                            }
                            System.arraycopy(SoftMixingClip.this.data, n4, array, i, n);
                            i += n;
                        }
                        SoftMixingClip.this._frameposition = n4 / SoftMixingClip.this.framesize;
                        return i - i;
                    }
                }
                int n8 = SoftMixingClip.this._frameposition * SoftMixingClip.this.framesize;
                int n9 = SoftMixingClip.this.bufferSize - n8;
                if (n9 == 0) {
                    return -1;
                }
                if (n > n9) {
                    n = n9;
                }
                System.arraycopy(SoftMixingClip.this.data, n8, array, i, n);
                SoftMixingClip.this._frameposition += n / SoftMixingClip.this.framesize;
                return n;
            }
        };
        this.open = false;
        this.frameposition = 0;
        this.frameposition_sg = false;
        this.active_sg = false;
        this.loopstart = 0;
        this.loopend = -1;
        this.active = false;
        this.loopcount = 0;
        this._active = false;
        this._frameposition = 0;
        this.loop_sg = false;
        this._loopcount = 0;
        this._loopstart = 0;
        this._loopend = -1;
    }
    
    @Override
    protected void processControlLogic() {
        this._rightgain = this.rightgain;
        this._leftgain = this.leftgain;
        this._eff1gain = this.eff1gain;
        this._eff2gain = this.eff2gain;
        if (this.active_sg) {
            this._active = this.active;
            this.active_sg = false;
        }
        else {
            this.active = this._active;
        }
        if (this.frameposition_sg) {
            this._frameposition = this.frameposition;
            this.frameposition_sg = false;
            this.afis = null;
        }
        else {
            this.frameposition = this._frameposition;
        }
        if (this.loop_sg) {
            this._loopcount = this.loopcount;
            this._loopstart = this.loopstart;
            this._loopend = this.loopend;
        }
        if (this.afis == null) {
            this.afis = AudioFloatInputStream.getInputStream(new AudioInputStream(this.datastream, this.format, -1L));
            if ((double)Math.abs(this.format.getSampleRate() - this.outputformat.getSampleRate()) > 1.0E-6) {
                this.afis = new SoftMixingDataLine.AudioFloatInputStreamResampler(this.afis, this.outputformat);
            }
        }
    }
    
    @Override
    protected void processAudioLogic(SoftAudioBuffer[] array) {
        if (this._active) {
            float[] array2 = array[0].array();
            float[] array3 = array[1].array();
            int size = array[0].getSize();
            int toIndex = size * this.in_nrofchannels;
            if (this.readbuffer == null || this.readbuffer.length < toIndex) {
                this.readbuffer = new float[toIndex];
            }
            try {
                int read = this.afis.read(this.readbuffer);
                if (read == -1) {
                    this._active = false;
                    return;
                }
                if (read != this.in_nrofchannels) {
                    Arrays.fill(this.readbuffer, read, toIndex, 0.0f);
                }
            }
            catch (IOException ex) {}
            int in_nrofchannels = this.in_nrofchannels;
            for (int i = 0, n = 0; i < size; ++i, n += in_nrofchannels) {
                int n2 = i;
                array2[n2] += this.readbuffer[n] * this._leftgain;
            }
            if (this.out_nrofchannels != 1) {
                if (this.in_nrofchannels == 1) {
                    for (int k = 0, n5 = 0; k < size; ++k, n5 += in_nrofchannels) {
                        int n6 = k;
                        array3[n6] += this.readbuffer[n5] * this._rightgain;
                    }
                }
                else {
                    for (int j = 0, n3 = 1; j < size; ++j, n3 += in_nrofchannels) {
                        int n4 = j;
                        array3[n4] += this.readbuffer[n3] * this._rightgain;
                    }
                }
            }
            if ((double)this._eff1gain > 2.0E-4) {
                float[] array4 = array[2].array();
                for (int l = 0, n7 = 0; l < size; ++l, n7 += in_nrofchannels) {
                    int n8 = l;
                    array4[n8] += this.readbuffer[n7] * this._eff1gain;
                }
                if (this.in_nrofchannels == 2) {
                    for (int n9 = 0, n10 = 1; n9 < size; ++n9, n10 += in_nrofchannels) {
                        int n11 = n9;
                        array4[n11] += this.readbuffer[n10] * this._eff1gain;
                    }
                }
            }
            if ((double)this._eff2gain > 2.0E-4) {
                float[] array5 = array[3].array();
                for (int n12 = 0, n13 = 0; n12 < size; ++n12, n13 += in_nrofchannels) {
                    int n14 = n12;
                    array5[n14] += this.readbuffer[n13] * this._eff2gain;
                }
                if (this.in_nrofchannels == 2) {
                    for (int n15 = 0, n16 = 1; n15 < size; ++n15, n16 += in_nrofchannels) {
                        int n17 = n15;
                        array5[n17] += this.readbuffer[n16] * this._eff2gain;
                    }
                }
            }
        }
    }
    
    @Override
    public int getFrameLength() {
        return this.bufferSize / this.format.getFrameSize();
    }
    
    @Override
    public long getMicrosecondLength() {
        return (long)((double)this.getFrameLength() * (1000000.0 / (double)this.getFormat().getSampleRate()));
    }
    
    @Override
    public void loop(int loopcount) {
        LineEvent lineEvent = null;
        synchronized (this.control_mutex) {
            if (this.isOpen()) {
                if (this.active) {
                    return;
                }
                this.active = true;
                this.active_sg = true;
                this.loopcount = loopcount;
                lineEvent = new LineEvent(this, LineEvent.Type.START, this.getLongFramePosition());
            }
        }
        if (lineEvent != null) {
            this.sendEvent(lineEvent);
        }
    }
    
    @Override
    public void open(AudioInputStream audioInputStream) throws LineUnavailableException, IOException {
        if (this.isOpen()) {
            throw new IllegalStateException("Clip is already open with format " + this.getFormat() + " and frame lengh of " + this.getFrameLength());
        }
        if (AudioFloatConverter.getConverter(audioInputStream.getFormat()) == null) {
            throw new IllegalArgumentException("Invalid format : " + audioInputStream.getFormat().toString());
        }
        if (audioInputStream.getFrameLength() != -1L) {
            byte[] b = new byte[(int)audioInputStream.getFrameLength() * audioInputStream.getFormat().getFrameSize()];
            int len = 512 * audioInputStream.getFormat().getFrameSize();
            int i = 0;
            while (i != b.length) {
                if (len > b.length - i) {
                    len = b.length - i;
                }
                int read2 = audioInputStream.read(b, i, len);
                if (read2 == -1) {
                    break;
                }
                if (read2 == 0) {
                    Thread.yield();
                }
                i += read2;
            }
            this.open(audioInputStream.getFormat(), b, 0, i);
        }
        else {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            byte[] array = new byte[512 * audioInputStream.getFormat().getFrameSize()];
            int read;
            while ((read = audioInputStream.read(array)) != -1) {
                if (read == 0) {
                    Thread.yield();
                }
                byteArrayOutputStream.write(array, 0, read);
            }
            this.open(audioInputStream.getFormat(), byteArrayOutputStream.toByteArray(), 0, byteArrayOutputStream.size());
        }
        return;
    }
    
    @Override
    public void open(AudioFormat format, byte[] original, int offset, int bufferSize) throws LineUnavailableException {
        synchronized (this.control_mutex) {
            if (this.isOpen()) {
                throw new IllegalStateException("Clip is already open with format " + this.getFormat() + " and frame lengh of " + this.getFrameLength());
            }
            if (AudioFloatConverter.getConverter(format) == null) {
                throw new IllegalArgumentException("Invalid format : " + format.toString());
            }
            if (bufferSize % format.getFrameSize() != 0) {
                throw new IllegalArgumentException("Buffer size does not represent an integral number of sample frames!");
            }
            if (original != null) {
                this.data = Arrays.copyOf(original, original.length);
            }
            this.offset = offset;
            this.bufferSize = bufferSize;
            this.format = format;
            this.framesize = format.getFrameSize();
            this.loopstart = 0;
            this.loopend = -1;
            this.loop_sg = true;
            if (!this.mixer.isOpen()) {
                this.mixer.open();
                this.mixer.implicitOpen = true;
            }
            this.outputformat = this.mixer.getFormat();
            this.out_nrofchannels = this.outputformat.getChannels();
            this.in_nrofchannels = format.getChannels();
            this.open = true;
            this.mixer.getMainMixer().openLine(this);
        }
    }
    
    @Override
    public void setFramePosition(int frameposition) {
        synchronized (this.control_mutex) {
            this.frameposition_sg = true;
            this.frameposition = frameposition;
        }
    }
    
    @Override
    public void setLoopPoints(int i, int j) {
        synchronized (this.control_mutex) {
            if (j != -1) {
                if (j < i) {
                    throw new IllegalArgumentException("Invalid loop points : " + i + " - " + j);
                }
                if (j * this.framesize > this.bufferSize) {
                    throw new IllegalArgumentException("Invalid loop points : " + i + " - " + j);
                }
            }
            if (i * this.framesize > this.bufferSize) {
                throw new IllegalArgumentException("Invalid loop points : " + i + " - " + j);
            }
            if (0 < i) {
                throw new IllegalArgumentException("Invalid loop points : " + i + " - " + j);
            }
            this.loopstart = i;
            this.loopend = j;
            this.loop_sg = true;
        }
    }
    
    @Override
    public void setMicrosecondPosition(long n) {
        this.setFramePosition((int)((double)n * ((double)this.getFormat().getSampleRate() / 1000000.0)));
    }
    
    @Override
    public int available() {
        return 0;
    }
    
    @Override
    public void drain() {
    }
    
    @Override
    public void flush() {
    }
    
    @Override
    public int getBufferSize() {
        return this.bufferSize;
    }
    
    @Override
    public AudioFormat getFormat() {
        return this.format;
    }
    
    @Override
    public int getFramePosition() {
        synchronized (this.control_mutex) {
            return this.frameposition;
        }
    }
    
    @Override
    public float getLevel() {
        return -1.0f;
    }
    
    @Override
    public long getLongFramePosition() {
        return (long)this.getFramePosition();
    }
    
    @Override
    public long getMicrosecondPosition() {
        return (long)((double)this.getFramePosition() * (1000000.0 / (double)this.getFormat().getSampleRate()));
    }
    
    @Override
    public boolean isActive() {
        synchronized (this.control_mutex) {
            return this.active;
        }
    }
    
    @Override
    public boolean isRunning() {
        synchronized (this.control_mutex) {
            return this.active;
        }
    }
    
    @Override
    public void start() {
        LineEvent lineEvent = null;
        synchronized (this.control_mutex) {
            if (this.isOpen()) {
                if (this.active) {
                    return;
                }
                this.active = true;
                this.active_sg = true;
                this.loopcount = 0;
                lineEvent = new LineEvent(this, LineEvent.Type.START, this.getLongFramePosition());
            }
        }
        if (lineEvent != null) {
            this.sendEvent(lineEvent);
        }
    }
    
    @Override
    public void stop() {
        LineEvent lineEvent = null;
        synchronized (this.control_mutex) {
            if (this.isOpen()) {
                if (!this.active) {
                    return;
                }
                this.active = false;
                this.active_sg = true;
                lineEvent = new LineEvent(this, LineEvent.Type.STOP, this.getLongFramePosition());
            }
        }
        if (lineEvent != null) {
            this.sendEvent(lineEvent);
        }
    }
    
    @Override
    public void close() {
        LineEvent lineEvent = null;
        synchronized (this.control_mutex) {
            if (!this.isOpen()) {
                return;
            }
            this.stop();
            lineEvent = new LineEvent(this, LineEvent.Type.CLOSE, this.getLongFramePosition());
            this.open = false;
            this.mixer.getMainMixer().closeLine(this);
        }
        if (lineEvent != null) {
            this.sendEvent(lineEvent);
        }
    }
    
    @Override
    public boolean isOpen() {
        return this.open;
    }
    
    @Override
    public void open() throws LineUnavailableException {
        if (this.data == null) {
            throw new IllegalArgumentException("Illegal call to open() in interface Clip");
        }
        this.open(this.format, this.data, this.offset, this.bufferSize);
        return;
    }
}
